@@ -39,7 +39,7 @@
 //!  3. Use a memory address as a very low randomness seed. If Address Space Layout Randomization
 //!     (ASLR) is supported by the operating system, this should be a pretty "random" value.
 //!
-//! It is highly recommended to use the `no_std` compatible `getrandom` feature to get high quality
+//! It is highly recommended using the `no_std` compatible `getrandom` feature to get high quality
 //! randomness seeds.
 //!
 //! The user can always create / update a generator with a user provided seed value.
@@ -48,6 +48,19 @@
 //! before creating the first random numbers, since the TLS instance is instantiated with a fixed
 //! value.
 //!
+//! ## SIMD
+//!
+//! The crate currently provides three generators that tries to use auto vectorization to speed up
+//! the generation of large amounts of random numbers.
+//!
+//!  * `Rng128` - This should be used when the processor has access to 128-bit SIMD (SSE2 / NEON).
+//!  * `Rng256` - This should be used when the processor has access to 256-bit SIMD (AVX2).
+//!  * `Rng512` - This should be used when the processor has access to 512-bit SIMD (AVX512).
+//!
+//! The nightly only feature `unstable_simd` uses the `core::simd` create to implement the SIMD.
+//! Users should test the available generators for their workload and verify if they can accelerate
+//! them using the SIMD functionality.
+//!
 //! ## Features
 //!
 //! The crate is `no_std` compatible.
@@ -58,7 +71,7 @@
 //!  * `getrandom` - Uses the `getrandom` crate to create a seed of high randomness. Enabled by default.
 //!  * `unstable_tls` - Uses the unstable `thread_local` feature of Rust nightly. Improves the call
 //!                     times to the thread local functions greatly.
-//!  * `unstable_simd` - Uses the unstable `std::simd` crate of Rust nightly to provide special SIMD
+//!  * `unstable_simd` - Uses the unstable `core::simd` crate of Rust nightly to provide special SIMD
 //!                      versions of the generator which can be used to create large amount of
 //!                      random data fast.
 #![warn(missing_docs)]
@@ -68,10 +81,12 @@
 #![cfg_attr(not(feature = "std"), no_std)]
 #![cfg_attr(docsrs, feature(doc_cfg))]
 
-#[cfg(feature = "unstable_simd")]
-mod simd;
+#[cfg(not(feature = "unstable_simd"))]
+mod stable_simd;
 #[cfg(all(feature = "tls", not(feature = "unstable_tls")))]
 mod stable_tls;
+#[cfg(feature = "unstable_simd")]
+mod unstable_simd;
 #[cfg(all(feature = "tls", feature = "unstable_tls"))]
 mod unstable_tls;
 
@@ -80,10 +95,12 @@ use core::{
     ops::{Bound, RangeBounds},
 };
 
-#[cfg(feature = "unstable_simd")]
-pub use simd::*;
+#[cfg(not(feature = "unstable_simd"))]
+pub use stable_simd::*;
 #[cfg(all(feature = "tls", not(feature = "unstable_tls")))]
 pub use stable_tls::*;
+#[cfg(feature = "unstable_simd")]
+pub use unstable_simd::*;
 #[cfg(all(feature = "tls", feature = "unstable_tls"))]
 pub use unstable_tls::*;
 
@@ -234,6 +251,10 @@ const fn split_mix_64(state: u64) -> [u64; 3] {
     let (y, state) = split_mix_64_round(state);
     let (z, _) = split_mix_64_round(state);
 
+    let x = if x == 0 { 1 } else { x };
+    let y = if y == 0 { 1 } else { y };
+    let z = if z == 0 { 1 } else { z };
+
     [x, y, z]
 }
 
@@ -287,9 +308,9 @@ impl Rng {
     /// Creates a fixed value to initialize the TLS.
     pub(crate) const fn fixed_tls() -> Self {
         Self {
-            x: Cell::new(0xad9da80ff4906d64),
-            y: Cell::new(0xd90576ebc62161ca),
-            z: Cell::new(0xbf0f8ca2e79b4817),
+            x: Cell::new(0xAD9DA80FF4906D64),
+            y: Cell::new(0xD90576EBC62161CA),
+            z: Cell::new(0xBF0F8CA2E79B4817),
             seed_source: Cell::new(SeedSource::Fixed),
         }
     }
@@ -299,9 +320,9 @@ impl Rng {
         let seed = split_mix_64(seed);
 
         Self {
-            x: Cell::new(seed[0] | 1),
-            y: Cell::new(seed[1] | 1),
-            z: Cell::new(seed[2] | 1),
+            x: Cell::new(seed[0]),
+            y: Cell::new(seed[1]),
+            z: Cell::new(seed[2]),
             seed_source: Cell::new(SeedSource::User),
         }
     }
@@ -332,9 +353,9 @@ impl Rng {
         let memory_address = self as *const _ as u64;
 
         let (seed, seed_source) = generate_seed(memory_address);
-        self.x.set(seed[0] | 1);
-        self.y.set(seed[1] | 1);
-        self.z.set(seed[2] | 1);
+        self.x.set(seed[0]);
+        self.y.set(seed[1]);
+        self.z.set(seed[2]);
         self.seed_source.set(seed_source)
     }
 
@@ -342,9 +363,9 @@ impl Rng {
     pub fn seed_with_64bit(&self, seed: u64) {
         let seed = split_mix_64(seed);
 
-        self.x.set(seed[0] | 1);
-        self.y.set(seed[1] | 1);
-        self.z.set(seed[2] | 1);
+        self.x.set(seed[0]);
+        self.y.set(seed[1]);
+        self.z.set(seed[2]);
         self.seed_source.set(SeedSource::User)
     }
 
@@ -844,4 +865,87 @@ mod tests {
 
         assert_ne!(org_bytes, bytes);
     }
+
+    #[test]
+    fn test_rng128() {
+        let mut rng = Rng128::from_seed_with_64bit([42, 43]);
+        rng.mix();
+
+        let rng0 = Rng::from_seed_with_64bit(42);
+        rng0.mix();
+
+        let rng1 = Rng::from_seed_with_64bit(43);
+        rng1.mix();
+
+        let res = rng.u64x2();
+
+        assert_eq!(res[0], rng0.u64());
+        assert_eq!(res[1], rng1.u64());
+    }
+
+    #[test]
+    fn test_rng256() {
+        let mut rng = Rng256::from_seed_with_64bit([42, 43, 44, 45]);
+        rng.mix();
+
+        let rng0 = Rng::from_seed_with_64bit(42);
+        rng0.mix();
+
+        let rng1 = Rng::from_seed_with_64bit(43);
+        rng1.mix();
+
+        let rng2 = Rng::from_seed_with_64bit(44);
+        rng2.mix();
+
+        let rng3 = Rng::from_seed_with_64bit(45);
+        rng3.mix();
+
+        let res = rng.u64x4();
+
+        assert_eq!(res[0], rng0.u64());
+        assert_eq!(res[1], rng1.u64());
+        assert_eq!(res[2], rng2.u64());
+        assert_eq!(res[3], rng3.u64());
+    }
+
+    #[test]
+    fn test_rng512() {
+        let mut rng = Rng512::from_seed_with_64bit([42, 43, 44, 45, 46, 47, 48, 49]);
+        rng.mix();
+
+        let rng0 = Rng::from_seed_with_64bit(42);
+        rng0.mix();
+
+        let rng1 = Rng::from_seed_with_64bit(43);
+        rng1.mix();
+
+        let rng2 = Rng::from_seed_with_64bit(44);
+        rng2.mix();
+
+        let rng3 = Rng::from_seed_with_64bit(45);
+        rng3.mix();
+
+        let rng4 = Rng::from_seed_with_64bit(46);
+        rng4.mix();
+
+        let rng5 = Rng::from_seed_with_64bit(47);
+        rng5.mix();
+
+        let rng6 = Rng::from_seed_with_64bit(48);
+        rng6.mix();
+
+        let rng7 = Rng::from_seed_with_64bit(49);
+        rng7.mix();
+
+        let res = rng.u64x8();
+
+        assert_eq!(res[0], rng0.u64());
+        assert_eq!(res[1], rng1.u64());
+        assert_eq!(res[2], rng2.u64());
+        assert_eq!(res[3], rng3.u64());
+        assert_eq!(res[4], rng4.u64());
+        assert_eq!(res[5], rng5.u64());
+        assert_eq!(res[6], rng6.u64());
+        assert_eq!(res[7], rng7.u64());
+    }
 }
